@@ -5,6 +5,7 @@ import (
 	"fmt"
 
 	corev1 "k8s.io/api/core/v1"
+	"k8s.io/apimachinery/pkg/api/equality"
 	"k8s.io/apimachinery/pkg/api/errors"
 	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
 	"k8s.io/apimachinery/pkg/util/intstr"
@@ -13,103 +14,72 @@ import (
 
 	configv1 "github.com/openshift/api/config/v1"
 	routev1 "github.com/openshift/api/route/v1"
+	"github.com/openshift/library-go/pkg/operator/resource/resourcemerge"
 )
 
 func (c *authOperator) handleRoute(ingress *configv1.Ingress) (*routev1.Route, *corev1.Secret, string, error) {
-	route, err := c.route.Get(targetName, metav1.GetOptions{})
+	expectedRoute := defaultRoute(ingress)
+
+	route, err := c.route.Get("oauth-openshift", metav1.GetOptions{})
 	if errors.IsNotFound(err) {
-		route, err = c.route.Create(defaultRoute(ingress))
+		route, err = c.route.Create(expectedRoute)
 	}
 	if err != nil {
 		return nil, nil, "FailedCreate", err
 	}
 
-	host := getCanonicalHost(route, ingress)
-	if len(host) == 0 {
-		// be careful not to print route.spec as it many contain secrets
-		return nil, nil, "FailedHost", fmt.Errorf("route is not available at canonical host %s: %+v", ingressToHost(ingress), route.Status.Ingress)
-	}
-
 	// assume it is unsafe to mutate route in case we go to a shared informer in the future
-	// this way everything else can just assume route.Spec.Host is correct
-	// note that we are not updating route.Spec.Host in the API - that value is nonsense to us
-	route = route.DeepCopy()
-	route.Spec.Host = host
+	existingCopy := route.DeepCopy()
+	modified := resourcemerge.BoolPtr(false)
+	resourcemerge.EnsureObjectMeta(modified, &existingCopy.ObjectMeta, expectedRoute.ObjectMeta)
 
-	if err := isValidRoute(route, ingress); err != nil {
-		// TODO remove this delete so that we do not lose the early creation timestamp of our route
-		// delete the route so that it is replaced with the proper one in next reconcile loop
-		klog.Infof("deleting invalid route: %#v", route)
-		opts := &metav1.DeleteOptions{Preconditions: &metav1.Preconditions{UID: &route.UID}}
-		if err := c.route.Delete(route.Name, opts); err != nil && !errors.IsNotFound(err) {
-			klog.Infof("failed to delete invalid route: %v", err)
+	// this guarantees that route.Spec.Host is set to the current canonical host
+	if *modified || !equality.Semantic.DeepEqual(existingCopy.Spec, expectedRoute.Spec) {
+		// be careful not to print route.spec as it many contain secrets
+		klog.Info("updating route")
+		existingCopy.Spec = expectedRoute.Spec
+		route, err = c.route.Update(existingCopy)
+		if err != nil {
+			return nil, nil, "FailedUpdate", err
 		}
-		return nil, nil, "InvalidRoute", err
 	}
 
-	routerSecret, err := c.secrets.Secrets(targetNamespace).Get(routerCertsLocalName, metav1.GetOptions{})
+	if ok := hasCanonicalHost(route, expectedRoute.Spec.Host); !ok {
+		// be careful not to print route.spec as it many contain secrets
+		return nil, nil, "FailedHost", fmt.Errorf("route is not available at canonical host %s: %+v", expectedRoute.Spec.Host, route.Status.Ingress)
+	}
+
+	routerSecret, err := c.secrets.Secrets("openshift-authentication").Get("v4-0-config-system-router-certs", metav1.GetOptions{})
 	if err != nil {
 		return nil, nil, "FailedRouterSecret", err
 	}
-	if len(routerSecret.Data) == 0 {
-		// be careful not to print the routerSecret even when it is empty
-		return nil, nil, "InvalidRouterSecret", fmt.Errorf("router secret %s/%s is empty", routerSecret.Namespace, routerSecret.Name)
-	}
 
 	return route, routerSecret, "", nil
 }
 
-func isValidRoute(route *routev1.Route, ingress *configv1.Ingress) error {
-	// TODO: return all errors at once
-	// TODO error when fields that should be empty are set
-
-	// get the expected settings from the default route
-	expectedRoute := defaultRoute(ingress)
-	expName := expectedRoute.Spec.To.Name
-	expPort := expectedRoute.Spec.Port.TargetPort.IntValue()
-	expTLSTermination := expectedRoute.Spec.TLS.Termination
-	expInsecureEdgeTerminationPolicy := expectedRoute.Spec.TLS.InsecureEdgeTerminationPolicy
-
-	if route.Spec.To.Name != expName {
-		return fmt.Errorf("route targets a wrong service - needs %s: %#v", expName, route)
-	}
-
-	if route.Spec.Port.TargetPort.IntValue() != expPort {
-		return fmt.Errorf("expected port '%d' for route: %#v", expPort, route)
-	}
-
-	if route.Spec.TLS == nil {
-		return fmt.Errorf("TLS needs to be configured for route: %#v", route)
-	}
-
-	if route.Spec.TLS.Termination != expTLSTermination {
-		return fmt.Errorf("route contains wrong TLS termination - '%s' is required: %#v", expTLSTermination, route)
-	}
-
-	if route.Spec.TLS.InsecureEdgeTerminationPolicy != expInsecureEdgeTerminationPolicy {
-		return fmt.Errorf("route contains wrong insecure termination policy - '%s' is required: %#v", expInsecureEdgeTerminationPolicy, route)
-	}
-
-	return nil
-}
-
 func defaultRoute(ingress *configv1.Ingress) *routev1.Route {
+	// emulates server-side defaulting as in https://github.com/openshift/openshift-apiserver/blob/master/pkg/route/apis/route/v1/defaults.go
+	// TODO: replace with server-side apply
+	var weightVal int32 = 100
+
 	return &routev1.Route{
 		ObjectMeta: defaultMeta(),
 		Spec: routev1.RouteSpec{
 			Host:      ingressToHost(ingress), // mimic the behavior of subdomain
 			Subdomain: "",                     // TODO once subdomain is functional, remove reliance on ingress config and just set subdomain=targetName
 			To: routev1.RouteTargetReference{
-				Kind: "Service",
-				Name: targetName,
+				Kind:   "Service",
+				Name:   "oauth-openshift",
+				Weight: &weightVal,
 			},
 			Port: &routev1.RoutePort{
-				TargetPort: intstr.FromInt(containerPort),
+				TargetPort: intstr.FromInt(6443),
 			},
 			TLS: &routev1.TLSConfig{
 				Termination:                   routev1.TLSTerminationPassthrough,
 				InsecureEdgeTerminationPolicy: routev1.InsecureEdgeTerminationPolicyRedirect,
 			},
+			WildcardPolicy: routev1.WildcardPolicyNone, // emulates server-side defaulting, see the link above
 		},
 	}
 }
@@ -121,8 +91,8 @@ func routerSecretToSNI(routerSecret *corev1.Secret) []configv1.NamedCertificate
 		out = append(out, configv1.NamedCertificate{
 			Names: []string{"*." + domain}, // ingress domain is always a wildcard
 			CertInfo: configv1.CertInfo{ // the cert and key are appended together
-				CertFile: routerCertsLocalMount + "/" + domain,
-				KeyFile:  routerCertsLocalMount + "/" + domain,
+				CertFile: "/var/config/system/secrets/v4-0-config-system-router-certs/" + domain,
+				KeyFile:  "/var/config/system/secrets/v4-0-config-system-router-certs/" + domain,
 			},
 		})
 	}
@@ -154,18 +124,17 @@ func routerSecretToCA(route *routev1.Route, routerSecret *corev1.Secret, ingress
 	return caData
 }
 
-func getCanonicalHost(route *routev1.Route, ingressConfig *configv1.Ingress) string {
-	host := ingressToHost(ingressConfig)
+func hasCanonicalHost(route *routev1.Route, canonicalHost string) bool {
 	for _, ingress := range route.Status.Ingress {
-		if ingress.Host != host {
+		if ingress.Host != canonicalHost {
 			continue
 		}
 		if !isIngressAdmitted(ingress) {
 			continue
 		}
-		return host
+		return true
 	}
-	return ""
+	return false
 }
 
 func isIngressAdmitted(ingress routev1.RouteIngress) bool {
@@ -178,5 +147,5 @@ func isIngressAdmitted(ingress routev1.RouteIngress) bool {
 }
 
 func ingressToHost(ingress *configv1.Ingress) string {
-	return targetName + "." + ingress.Spec.Domain
+	return "oauth-openshift." + ingress.Spec.Domain
 }
